// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
#ifdef MODBUS_SUPPORT_ENABLED
#ifndef _MBHARDWARE_H_
#define _MBHARDWARE_H_

#include <silecs-communication/interface/communication/ietype.h>
#include <modbus.h>

/*----------------------------------------------------------*/
/* Time funtion
 * IeMdbSetTime: Convert time_t epoch date to PLC SCHNEIDER
 * format.
 *
 * Details:
 * PLC SCHNEIER format is coded on 8 bytes BCD
 * format:
 *
 *  0      1      2      3      4      5     6     7
 *  SC     --     HH     MN     MM     DD    YY    YY
 *
 *  byte 1 is used by gateway to synchronize time setting.
 */
void IeMdbSetTime(unsigned char *dt, time_t epoch); // to PLC

/*----------------------------------------------------------*/
/* Time funtion
 * IeMdbGetTime: Convert PLC SCHNEIDER format to lynxOS
 * time_t format.
 *
 * Details:
 * PLC SCHNEIER format is coded on 8 bytes BCD
 * format:
 *
 *  0      1        2      3      4      5     6     7
 *  SC     SC/100   HH     MN     MM     DD    YY    YY
 *
 */
double IeMdbGetTime(unsigned char *dt); // from PLC

#endif /* _MBHARDWARE_H_ */
#endif //MODBUS_SUPPORT_ENABLED
